#! /usr/bin/env python

import sys
from optparse import OptionParser
import random

parser = OptionParser()
parser.add_option('-s', '--seed', help='the random seed',
                  default=0, action='store', type='int', dest='seed')
parser.add_option('-o', '--output_file', help='output file name (.c fragment)',
                  default='main.c', action='store', type='string', dest='output_file')
parser.add_option('-e', '--exec_file', help='(.c fragment)',
                  default='main', action='store', type='string', dest='exec_file')
parser.add_option('-n', '--num_questions', help='number of questions',
                  default=5, action='store', type='int', dest='num_questions')
parser.add_option('-m', '--max_value', help='max value',
                  default=10, action='store', type='int', dest='max_value')
parser.add_option('-p', '--prob_continue', help='chance to make expr longer',
                  default=0.3, action='store', type='float', dest='prob_continue')
parser.add_option('-c', help='compute answers for me', action='store_true',
                  default=False, dest='solve')

(options, args) = parser.parse_args()

print ''
print 'The following is a list of C boolean comparison statements, using'
print 'the six basic boolean operators: >, <, >=, <=, !=, and == which'
print 'are less than, greater than, less than or equal, greater than or equal,'
print 'not equal, and equal, respectively. Use your knowledge of the rules of'
print 'precedence to calculate the values for each expression, and then'
print 'use the -c flag to see if you were right. You can specify different'
print 'random seeds (-s <value>) to get different randomly-generated'
print 'expressions to test your knowledge; use -h to see more options.'
print ''


seed = int(options.seed)
output_file = str(options.output_file)
exec_file = str(options.exec_file)
num_questions = int(options.num_questions)
prob_continue = float(options.prob_continue)
max_value = int(options.max_value)
solve = options.solve

random.seed(options.seed)

paren_level = 0

def make_varname():
    global var_counter
    var_length = 1 + var_counter / 26
    var_char = chr(var_counter % 26 + 97)
    var_counter += 1
    return var_char * var_length;

def make_op():
    rvalue = random.random()
    if rvalue < 0.16:
        return '<'
    elif rvalue < 0.32:
        return '>'
    elif rvalue < 0.48:
        return '<='
    elif rvalue < 0.64:
        return '>='
    elif rvalue < 0.80:
        return '!='
    else:
        return '=='

def make_number():
    global max_value
    return str(int(random.random() * max_value) + 1)

def add_paren():
    global paren_level
    add_one = random.random() < 0.2
    if add_one:
        paren_level += 1
        return True
    return False

def close_paren():
    global paren_level
    if paren_level == 0:
        return
    sub_one = random.random() < 0.8
    if sub_one:
        paren_level -= 1
        return True
    return False

# now start spitting out assignment statements
questions = []
max_question_len = 0
for i in range(num_questions):
    out_str = make_number()
    curr_prob_continue = 1.0
    while random.random() < curr_prob_continue:
        if add_paren():
            out_str += ' %s (%s' % (make_op(), make_number())
        elif close_paren():
            out_str += ') %s %s' % (make_op(), make_number())
        else:
            out_str += ' %s %s' % (make_op(), make_number())
        curr_prob_continue = prob_continue
    while paren_level > 0:
        out_str += ')'
        paren_level -= 1
    if len(out_str) > max_question_len:
        max_question_len = len(out_str)
    questions.append(out_str)

fd = open(output_file, 'w')
fd.write('#include <stdio.h>\n')
fd.write('\n')
fd.write('int main(int argc, char *argv[]) {\n')
for q in questions:
    if solve:
        fd.write('  printf("%s = %%d\\n", (%s));\n' % (q, q))
    else:
        fd.write('  printf("%s = ?\\n");\n' % q)
fd.write('  return 0;\n')
fd.write('}\n')
fd.close()

import subprocess
rc = subprocess.call(['/usr/bin/gcc', '-o', exec_file, '-m32', output_file])

if rc == 0:
    rc = subprocess.call(['./%s' % exec_file])

clean_up = True
if clean_up:
    subprocess.call(['rm', '-f', exec_file])

print ''
