#! /usr/bin/env python

import sys
from optparse import OptionParser
import random

parser = OptionParser()
parser.add_option('-s', '--seed', help='the random seed',
                  default=0, action='store', type='int', dest='seed')
parser.add_option('-n', '--num_questions', help='number of questions',
                  default=5, action='store', type='int', dest='num_questions')
parser.add_option('-m', '--max_value', help='max value',
                  default=10, action='store', type='int', dest='max_value')
parser.add_option('-p', '--prob_continue', help='chance to make expr longer',
                  default=0.65, action='store', type='float', dest='prob_continue')
parser.add_option('-N', '--prob_negative', help='chance to make integer negative',
                  default=0.0, action='store', type='float', dest='prob_negative')
parser.add_option('-P', '--paren_chance', help='chance to add a paren',
                  default=0.25, action='store', type='float', dest='paren_chance')
parser.add_option('-c', help='compute answers for me', action='store_true',
                  default=False, dest='solve')

(options, args) = parser.parse_args()

print ''
print 'The following is a list of C expressions over integers, using'
print 'the five basic integer math operators: *, /, %, +, and -, which'
print 'are multiplication, (integer) division, remainder, addition, and'
print 'subtraction, respectively. Use your knowledge of the rules of'
print 'precedence to calculate the values for each expression, and then'
print 'use the -c flag to see if you were right. You can specify different'
print 'random seeds (-s <value>) to get different randomly-generated'
print 'expressions to test your knowledge; use -h to see more options.'
print ''


seed = int(options.seed)
num_questions = int(options.num_questions)
prob_continue = float(options.prob_continue)
prob_negative = float(options.prob_negative)
paren_chance = float(options.paren_chance)
max_value = int(options.max_value)
solve = options.solve

random.seed(options.seed)

paren_level = 0

def make_varname():
    global var_counter
    var_length = 1 + var_counter / 26
    var_char = chr(var_counter % 26 + 97)
    var_counter += 1
    return var_char * var_length;

def make_op():
    rvalue = random.random()
    if rvalue < 0.20:
        return '+'
    elif rvalue < 0.40:
        return '-'
    elif rvalue < 0.60:
        return '*'
    elif rvalue < 0.80:
        return '*'
    else:
        return '%'

def make_number():
    global max_value
    global prob_negative
    if random.random() < prob_negative:
        return '-' + str(int(random.random() * max_value) + 1)
    else:
        return str(int(random.random() * max_value) + 1)

def add_paren():
    global paren_level
    global paren_chance
    add_one = random.random() < paren_chance
    if add_one:
        paren_level += 1
        return True
    return False

def close_paren():
    global paren_level
    global paren_chance
    if paren_level == 0:
        return False
    sub_one = random.random() < paren_chance
    if sub_one:
        paren_level -= 1
        return True
    return False

# now start spitting out assignment statements
questions = []
debug = []
max_question_len = 0
for i in range(num_questions):
    if add_paren():
        out_str = '(%s' % (make_number())
        dont_close = True
    else:
        out_str = make_number()
        dont_close = False
    curr_prob_continue = 1.0

    debug_str = ''

    while random.random() < curr_prob_continue:
        if add_paren():
            debug_str += 'add_paren(%d) ' % paren_level
            out_str += ' %s (%s' % (make_op(), make_number())
            dont_close = True
        elif not dont_close and close_paren():
            debug_str += 'close_paren(%d) ' % paren_level
            out_str += ') %s %s' % (make_op(), make_number())
            dont_close = False
        else:
            debug_str += 'add_op '
            out_str += ' %s %s' % (make_op(), make_number())
            dont_close = False
        curr_prob_continue = prob_continue

    # make sure to add right number of right-parens at end
    debug.append((out_str, debug_str + str(paren_level)))
    while paren_level > 0:
        out_str += ')'
        paren_level -= 1
    if len(out_str) > max_question_len:
        max_question_len = len(out_str)
    questions.append(out_str)

# i = 0
for q in questions:
    # print 'DEBUG', debug[i]
    # i += 1
    print q,
    if solve:
        try:
            result = eval(q)
        except ZeroDivisionError:
            print '-> divide by zero (error)'
        else:
            print '=', result
    else:
        print '= ?'

print ''
