#! /usr/bin/env python

import sys
from optparse import OptionParser
import random


parser = OptionParser()
parser.add_option('-s', '--seed', help='the random seed',
                  default=0, action='store', type='int', dest='seed')
parser.add_option('-n', '--num_questions', help='number of questions',
                  default=5, action='store', type='int', dest='num_questions')
parser.add_option('-S', '--memory_size', help='size of memory',
                  default=16, action='store', type='int', dest='memory_size')
parser.add_option('-m', '--max_value', help='max value',
                  default=256, action='store', type='int', dest='max_value')
parser.add_option('-c', help='compute answers for me', action='store_true',
                  default=False, dest='solve')

(options, args) = parser.parse_args()

seed = int(options.seed)
num_questions = int(options.num_questions)
memory_size = int(options.memory_size)
max_value = int(options.max_value)
solve = options.solve

random.seed(options.seed)

print ''
print 'This is just a very simple program that shows how you can access a'
print 'byte-addressable memories. Basically, there are two primitives to access'
print 'memory, load and store. The load instruction takes an address and returns'
print 'the value stored at that address in memory. In this simple example, we '
print 'assume that there is only a single byte stored there (from 0-255). A store'
print 'instruction takes an address and a value and simply updates memory at that'
print 'address to the new value. The program generates a random series of load and'
print 'store operations; you should see if you can figure out the return value from'
print 'each load, and the final contents of memory after all the loads and stores'
print 'have completed. Use -s <seed> to generate different problems, and -c to have'
print 'the computer solve the problem for you to check answers.'
print ''


print ''    
print 'Initial Memory Contents:'
def memory_dump(memory, memory_size):
    print '  Address [ Contents ]'
    for i in range(memory_size):
        print '   %5d  [   %4s   ]' % (i, str(memory[i]))
    

# make memory
memory = {}
for i in range(memory_size):
    memory[i] = int(random.random() * max_value)

memory_dump(memory, memory_size)

print ''
print 'Memory Reference Trace:'
for i in range(num_questions):
    addr = int(random.random() * memory_size)
    is_load = random.random() < 0.5
    if is_load:
        if solve:
            print '   load [address:%4d]            -> %d' % (addr, memory[addr])
        else:
            print '   load [address:%4d]            -> ?' % addr
    else:
        value = int(random.random() * max_value)
        if solve:
            print '  store [address:%4d value:%4d]' % (addr, value)
        else:
            print '  store [address:%4d value:%4d]' % (addr, value)
        memory[addr] = value

if solve:
    print ''
    print 'Final Memory Contents:'
    memory_dump(memory, memory_size)

print ''
