#! /usr/bin/env python

import sys
from optparse import OptionParser
import random

parser = OptionParser()
parser.add_option('-s', '--seed', help='the random seed',
                  default=0, action='store', type='int', dest='seed')
parser.add_option('-e', '--exec_file', help='(.c fragment)',
                  default='main', action='store', type='string', dest='exec_file')
parser.add_option('-v', '--num_variables', help='number of variables',
                  default=5, action='store', type='int', dest='num_variables')
parser.add_option('-n', '--num_questions', help='number of questions',
                  default=5, action='store', type='int', dest='num_questions')
parser.add_option('-m', '--max_value', help='max value',
                  default=10, action='store', type='int', dest='max_value')
parser.add_option('-p', '--prob_continue', help='chance to make expr longer',
                  default=0.8, action='store', type='float', dest='prob_continue')
parser.add_option('-C', '--cleanup', help='remove executable when done',
                  action='store_false', default=True, dest='cleanup')
parser.add_option('-c', help='compute answers for me', action='store_true',
                  default=False, dest='solve')
(options, args) = parser.parse_args()

print ''
print 'This program generates a number (determined by -v) of C integers'
print 'and then sets them to randomly-generated expressions. Your job is'
print 'to step through the expressions, one at a time, from top to bottom'
print 'and to determine the value each variable takes along the way. Use'
print 'the -c flag to show the answers, the -s flag (with a value) to '
print 'generate different random expressions, and the -h flag to see all'
print 'other options.'
print ''
print 'Here is the C program:'
print ''

num_variables = int(options.num_variables)
num_questions = int(options.num_questions)
exec_file = str(options.exec_file)
max_value = int(options.max_value)
prob_continue = float(options.prob_continue)
seed = int(options.seed)
solve = options.solve
clean_up = options.cleanup

random.seed(options.seed)

var_counter = 0

def make_varname():
    global var_counter
    var_length = 1 + var_counter / 26
    var_char = chr(var_counter % 26 + 97)
    var_counter += 1
    return var_char * var_length;

def make_op():
    rvalue = random.random()
    if rvalue < 0.2:
        return '+'
    elif rvalue < 0.4:
        return '-'
    elif rvalue < 0.6:
        return '*'
    elif rvalue < 0.8:
        return '%'
    else:
        return '/'

def make_number_or_variable():
    global max_value
    global defined_vars
    if len(defined_vars) > 1 and random.random() > 0.5:
        index = int(random.random() * len(defined_vars))
        return defined_vars[index]
    else:
        return str(int(random.random() * max_value) + 1)

created_vars = []
defined_vars = []

# make variables first
py_out = ''
for i in range(num_variables):
    var_name = make_varname()
    assert(var_name not in created_vars)
    created_vars.append(var_name)
    print 'int %s;' % var_name
    out_str = 'int %s\n' % var_name

# now start spitting out assignment statements
first = True

if num_questions > 100:
    print 'Warning: limiting to 100 questions'
    num_questions = 100

for i in range(num_questions):
    var_index = int(random.random() * num_variables)
    var_name = created_vars[var_index]
    # OLD: direct eval
    # out_str = '%s = %s' % (var_name, make_number_or_variable())
    out_str = '%s' % (make_number_or_variable())
    while random.random() < prob_continue:
        out_str += ' %s %s' % (make_op(), make_number_or_variable())
    out_str += ''
    if var_name not in defined_vars:
        defined_vars.append(var_name)

    if first and not solve:
        py_out += 'print "%s = %s;", \n' % (var_name, out_str)
        first = False
    else:
        py_out += 'print "%s = %s;", \n' % (var_name, out_str)

    py_out += 'try:\n'
    py_out += '    result = eval("%s")\n' % out_str
    py_out += 'except ZeroDivisionError:\n'
    py_out += '    %s = 1\n' % var_name
    if solve:
        py_out += '    print " ERROR: div by zero"\n'
        py_out += '    exit(1)\n' 
    py_out += 'else:\n'
    py_out += '    %s = result\n' % var_name
    # OLD: direct eval
    # py_out += out_str + '\n'
    
    if solve:
        pad_to = 50
        cur_len = len(out_str)
        pad_print = ''
        while cur_len < pad_to:
            pad_print += ' '
            cur_len += 1
        py_out += 'print "%s", %s\n' % (pad_print, var_name)
    else:
        py_out += 'print ""\n'

print ''
if solve:
    py_out += 'print "\\nFinal values:"\n'
else:
    py_out += 'print "\\nWhat are the final values:"\n'
for v in defined_vars:
    if solve:
        py_out += 'print "%s =", %s\n' % (v, v)
    else:
        py_out += 'print "%s = ?"\n' % (v)

fd = open(exec_file, 'w')
fd.write(py_out)
fd.close()

import subprocess
rc = subprocess.call(['python', exec_file])

if clean_up:
    subprocess.call(['rm', '-f', exec_file])


